/* 
 * Distort - BeOS screensaver based on the xscreensaver Distort
 * BeOS rewrite, copyright © 1999 by John Yanarella (yanarejm@muohio.edu)
 *
 * version 1.1 (March 29th, 2000)
 *    - updated to compile under R5
 *
 * version 1.0 (November 17th, 1999)
 *    - original release
 *
 * Although the BeOS version involved a great deal of rewriting,
 * it's still covered by this license:
 * 
 * distort by Jonas Munsin (jmunsin@iki.fi) and Jamie Zawinski (jwz@jwz.org)
 *
 * xscreensaver, Copyright (c) 1992, 1993, 1994, 1996, 1997, 1998
 * Jamie Zawinski <jwz@jwz.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 *
 */

#ifndef DISTORT_H
#define DISTORT_H

#include <Locker.h>
#include <ScreenSaver.h>
#include <Bitmap.h>

struct coo {
	int x;
	int y;
	int xmove, ymove;
};

class Distort : public BScreenSaver, public BLocker {
 public:
	Distort(BMessage *msg, image_id id);
	virtual ~Distort();
	
	virtual void StartConfig(BView *view);
	virtual status_t StartSaver(BView *view, bool preview);
	virtual void StopSaver();
	virtual void Draw(BView *view, int32 frame);
	
	status_t SaveState(BMessage *msg) const;
	void RestoreState(BMessage *msg);
	
 private:
	friend class DistortConfigView;
	
	BBitmap *m_icon;
	bool m_preview;
	image_id m_image;

	struct coo m_xy_coo;
	int m_delay, m_radius, m_speed, m_type;
	// m_type: 0) vortex, 1) blackhole, 2) magnify, 3) reflect, 4) custom
	float m_twist;

	int m_width, m_height;
	color_space m_colorspace;
	BBitmap *m_orig_map, *m_buffer_map;
	int8 *m_orig8, *m_buf8;
	int16 *m_orig16, *m_buf16;
	int32 *m_orig32, *m_buf32;
	
	int ***m_from;
	void allocate_lens();
	void deallocate_lens();
	void make_round_lens(int radius, int loop);
	void move_lens();	
	void reflect_draw(BView* view);
	void plain_draw(BView* view);
};

#define RADIUS_CHANGED 	'mRad'
#define SPEED_CHANGED 	'mSpd'
#define TWIST_CHANGED	'mTws'
#define TYPE_CHANGED	'mTyp'

class DistortConfigView : public BView 
{
 public:
	DistortConfigView(BRect frame, Distort *s);
	virtual void AttachedToWindow(void);
	virtual void MessageReceived(BMessage *message);
	
 private:
	Distort *m_screensaver;	
	BSlider	*m_radius_slider, *m_speed_slider, *m_twist_slider;
	BMenu *m_lens_menu;
};

#endif // DISTORT_H